local Critline = Critline
local L = LibStub("AceLocale-3.0"):GetLocale("Critline")
local templates = Critline.templates

local checks = {
	GUILD = IsInGuild,
	PARTY = function() return GetNumPartyMembers() > 0 end,
	RAID = function() return GetNumRaidMembers() > 0 end,
}

local addon = templates:CreateConfigFrame("AutoAnnounce", "Critline", true)

do
	local enabled = templates:CreateCheckButton(addon)
	enabled:SetPoint("TOPLEFT", addon.title, "BOTTOMLEFT", -2, -16)
	enabled:SetText(L["Enabled"])
	enabled.setting = "enabled"
	enabled.module = addon
	enabled.db = "db"
	enabled.func = function(self)
		if self:GetChecked() then
			Critline.RegisterCallback(addon, "NewRecord")
		else
			Critline.UnregisterCallback(addon, "NewRecord")
		end
	end
	addon.enabled = enabled

	local function onClick(self)
		self.owner:SetSelectedValue(self.value)
		addon.db.profile.channel = self.value
		local target = addon.target
		if self.value == "WHISPER" or self.value == "CHANNEL" then
			target:Show()
			target:SetFocus()
		else
			target:Hide()
		end
	end

	local channels = {
		"SAY",
		"GUILD",
		"PARTY",
		"RAID",
		"WHISPER",
		"CHANNEL",
	}

	local channel = templates:CreateDropDownMenu("CritlineAutoAnnounceChannel", addon, nil, _G)
	channel:SetFrameWidth(120)
	channel:SetPoint("TOPLEFT", enabled, "BOTTOMLEFT", -12, -16)
	channel:SetSelectedValue("SAY")
	channel.initialize = function(self)
		for i, v in ipairs(channels) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = _G[v]
			info.value = v
			info.func = onClick
			info.owner = self
			UIDropDownMenu_AddButton(info)
		end
	end
	addon.channel = channel

	local target = templates:CreateEditBox(addon)
	target:SetWidth(134)
	target:SetPoint("TOP", channel, "BOTTOM", 0, -8)
	target:SetScript("OnEnterPressed", function(self)
		local text = self:GetText():trim()
		local channel = channel:GetSelectedValue()
		if channel == "WHISPER" then
			if text == "" then
				Critline:Message(L["Invalid player name."])
				return
			end
		elseif channel == "CHANNEL" then
			text = GetChannelName(text)
			if text == 0 then
				Critline:Message(L["Invalid channel. Please enter a valid channel name or ID."])
				return
			end
		end
		addon.db.profile.target = text
		self:ClearFocus()
	end)
	target:SetScript("OnEscapePressed", target.ClearFocus)
	target:Hide()
	addon.target = target
end


function addon:LoadSettings()
	local enabled = self.db.profile.enabled
	if enabled then
		Critline.RegisterCallback(self, "NewRecord")
	else
		Critline.UnregisterCallback(self, "NewRecord")
	end
	addon.enabled:SetChecked(enabled)
	addon.channel:SetSelectedValue(self.db.profile.channel)
	addon.target:SetText(self.db.profile.target)
end


function addon:NewRecord(event, tree, spellID, periodic, amount, crit)
	local channel = self.db.profile.channel
	if checks[channel] and not checks[channel]() then
		return
	end
	SendChatMessage(format(L["New %s%s record - %d"], crit and L["critical "] or "", Critline:GetSpellName(spellID), amount), channel, nil, self.db.profile.target)
end


local defaults = {
	profile = {
		enabled = true,
		channel = "SAY",
		target = "",
	}
}

addon.db = Critline.db:RegisterNamespace("AutoAnnounce", defaults)
addon:LoadSettings()
Critline.RegisterCallback(addon, "SettingsLoaded", "LoadSettings")